@@ -1,16 +1,17 @@
-""" :run
-"""
+"""Run subcommand implementation."""
 import curses
-import datetime
 import json
 import logging
 import os
 import re
 import shlex
 import shutil
+import time
 import uuid
 
 from math import floor
+from operator import itemgetter
+from pathlib import Path
 from queue import Queue
 from typing import Any
 from typing import Callable
@@ -18,27 +19,35 @@ from typing import Dict
 from typing import List
 from typing import Optional
 from typing import Tuple
-from typing import Union
 
-from . import run_action
-from . import _actions as actions
-
-from ..runner.api import CommandRunnerAsync
-from ..app import App
+from ..action_base import ActionBase
+from ..action_defs import RunStdoutReturn
 from ..app_public import AppPublic
 from ..configuration_subsystem import ApplicationConfiguration
+from ..configuration_subsystem import to_effective
+from ..configuration_subsystem import to_sources
+from ..content_defs import ContentView
+from ..content_defs import SerializationFormat
+from ..runner import CommandAsync
 from ..steps import Step
+from ..ui_framework import CursesLine
 from ..ui_framework import CursesLinePart
 from ..ui_framework import CursesLines
 from ..ui_framework import Interaction
 from ..ui_framework import dict_to_form
 from ..ui_framework import form_to_dict
+from ..ui_framework import nonblocking_notification
 from ..ui_framework import warning_notification
-
-
-from ..utils import abs_user_path
-from ..utils import human_time
-from ..utils import remove_ansi
+from ..utils.functions import abs_user_path
+from ..utils.functions import human_time
+from ..utils.functions import is_jinja
+from ..utils.functions import now_iso
+from ..utils.functions import remove_ansi
+from ..utils.functions import round_half_up
+from ..utils.serialize import serialize_write_file
+from . import _actions as actions
+from . import run_action
+from .stdout import Action as stdout_action
 
 
 RESULT_TO_COLOR = [
@@ -46,21 +55,23 @@ RESULT_TO_COLOR = [
     ("(?i)^ok$", 10),
     ("(?i)^ignored$", 13),
     ("(?i)^skipped$", 14),
-    ("(?i)^in_progress$", 8),
+    ("(?i)^in progress$", 8),
 ]
 
 get_color = lambda word: next(  # noqa: E731
-    (x[1] for x in RESULT_TO_COLOR if re.match(x[0], word)), 0
+    (x[1] for x in RESULT_TO_COLOR if re.match(x[0], word)),
+    0,
 )
 
 
 def color_menu(_colno: int, colname: str, entry: Dict[str, Any]) -> Tuple[int, int]:
     # pylint: disable=too-many-branches
-    """Find matching color for word
+    """Find matching color for word.
 
-    :param colname: A word to match
+    :param colname: The column name
+    :param entry: The menu entry
+    :returns: The color and decoration
     """
-
     colval = entry[colname]
     color = 0
     decoration = 0
@@ -84,7 +95,7 @@ def color_menu(_colno: int, colname: str, entry: Dict[str, Any]) -> Tuple[int, i
             decoration = curses.A_BOLD
 
     elif "task" in entry:
-        if entry["__result"].lower() == "__in_progress":
+        if entry["__result"].lower() == "in progress":
             color = get_color(entry["__result"])
         elif colname in ["__result", "__host", "__number", "__task", "__task_action"]:
             color = get_color(entry["__result"])
@@ -99,26 +110,23 @@ def color_menu(_colno: int, colname: str, entry: Dict[str, Any]) -> Tuple[int, i
     return color, decoration
 
 
-def content_heading(obj: Any, screen_w: int) -> Union[CursesLines, None]:
-    """create a heading for some piece fo content showing
+def content_heading(obj: Any, screen_w: int) -> Optional[CursesLines]:
+    """Create a heading for some piece of content showing.
 
     :param obj: The content going to be shown
     :param screen_w: The current screen width
-    :return: The heading
+    :returns: The heading
     """
-
     if isinstance(obj, dict) and "task" in obj:
-        heading = []
-        detail = "PLAY [{play}:{tnum}] ".format(play=obj["play"], tnum=obj["__number"])
-        stars = "*" * (screen_w - len(detail))
-        heading.append(
-            tuple([CursesLinePart(column=0, string=detail + stars, color=0, decoration=0)])
+        detail = f"Play name: {obj['play']}:{obj['__number']}"
+
+        line_1 = CursesLine(
+            (CursesLinePart(column=0, string=detail, color=0, decoration=0),),
         )
 
-        detail = "TASK [{task}] ".format(task=obj["task"])
-        stars = "*" * (screen_w - len(detail))
-        heading.append(
-            tuple([CursesLinePart(column=0, string=detail + stars, color=0, decoration=0)])
+        detail = f"Task name: {obj['task']}"
+        line_2 = CursesLine(
+            (CursesLinePart(column=0, string=detail, color=0, decoration=0),),
         )
 
         if obj["__changed"] is True:
@@ -133,26 +141,22 @@ def content_heading(obj: Any, screen_w: int) -> Union[CursesLines, None]:
         else:
             msg = ""
 
-        string = "{res}: [{host}] {msg}".format(res=res, host=obj["__host"], msg=msg)
+        string = f"{res}: {obj['__host']} {msg}"
         string = string + (" " * (screen_w - len(string) + 1))
-        heading.append(
-            tuple(
-                [
-                    CursesLinePart(
-                        column=0,
-                        string=string,
-                        color=color,
-                        decoration=curses.A_UNDERLINE,
-                    )
-                ]
-            )
+        line_3 = CursesLine(
+            (CursesLinePart(column=0, string=string, color=color, decoration=curses.A_UNDERLINE),),
         )
-        return tuple(heading)
+
+        return CursesLines((line_1, line_2, line_3))
     return None
 
 
 def filter_content_keys(obj: Dict[Any, Any]) -> Dict[Any, Any]:
-    """when showing content, filter out some keys"""
+    """Filter out some keys when showing collection content.
+
+    :param obj: The object from which keys should be removed
+    :returns: The object with keys removed
+    """
     return {k: v for k, v in obj.items() if not (k.startswith("_") or k.endswith("uuid"))}
 
 
@@ -164,7 +168,7 @@ PLAY_COLUMNS = [
     "__failed",
     "__skipped",
     "__ignored",
-    "__in_progress",
+    "__in progress",
     "__task_count",
     "__progress",
 ]
@@ -181,10 +185,10 @@ TASK_LIST_COLUMNS = [
 
 
 @actions.register
-class Action(App):
+class Action(ActionBase):
 
     # pylint: disable=too-many-instance-attributes
-    """:run"""
+    """Run command implementation."""
 
     KEGEX = r"""(?x)
             ^
@@ -192,54 +196,64 @@ class Action(App):
             (\s(?P<params_run>.*))?)
             $"""
 
-    def __init__(
-        self,
-        args: ApplicationConfiguration,
-        play_columns: List = PLAY_COLUMNS,
-        task_list_columns: List = TASK_LIST_COLUMNS,
-        content_key_filter: Callable = filter_content_keys,
-    ):
-        # pylint: disable=dangerous-default-value
-        # for display purposes use the 4: of the uuid
+    def __init__(self, args: ApplicationConfiguration):
+        """Initialize the ``:run`` action.
+
+        :param args: The current settings for the application
+        """
         super().__init__(args=args, logger_name=__name__, name="run")
 
         self._subaction_type: str
         self._msg_from_plays: Tuple[Optional[str], Optional[int]] = (None, None)
         self._queue: Queue = Queue()
-        self.runner: CommandRunnerAsync
+        self.runner: CommandAsync
         self._runner_finished: bool
         self._auto_scroll = False
+        #: Flag when the first message is received from runner
+        self._first_message_received: bool = False
 
         self._plays = Step(
             name="plays",
-            tipe="menu",
-            columns=play_columns,
+            step_type="menu",
+            columns=PLAY_COLUMNS,
             value=[],
             show_func=self._play_stats,
             select_func=self._task_list_for_play,
         )
-        self._task_list_columns = task_list_columns
-        self._content_key_filter = content_key_filter
+        self._task_list_columns: List[str] = TASK_LIST_COLUMNS
+        self._content_key_filter: Callable = filter_content_keys
+        self._task_cache: Dict[str, str] = {}
+        """Task name storage from playbook_on_start using the task uuid as the key"""
 
     @property
     def mode(self):
-        """if mode == stdout and playbook artifact creation is enabled
-        run in interactive mode, but print stdout"""
+        """Determine the mode and if playbook artifact creation is enabled.
+
+        If so, run in interactive mode, but print stdout.
+
+        :returns: If stdout and artifact creation, return a str statement as such, else return mode
+        """
         if all(
             (
                 self._args.mode == "stdout",
                 self._args.playbook_artifact_enable,
                 self._args.app != "replay",
-            )
+            ),
         ):
             return "stdout_w_artifact"
         return self._args.mode
 
-    def run_stdout(self) -> int:
-        """Run in oldschool mode, just stdout."""
+    def run_stdout(self) -> RunStdoutReturn:
+        """Execute the ``inventory`` request for mode stdout.
+
+        :returns: The return code from the runner invocation, along with a message to review the
+            logs if not 0
+        """
         if self._args.app == "replay":
             successful: bool = self._init_replay()
-            return 0 if successful else 1
+            if successful:
+                return RunStdoutReturn(message="", return_code=0)
+            return RunStdoutReturn(message="Please review the log for errors.", return_code=1)
 
         self._logger.debug("playbook requested in interactive mode")
         self._subaction_type = "playbook"
@@ -252,18 +266,24 @@ class Action(App):
                     self.write_artifact()
                 self._logger.debug("runner finished")
                 break
-        return self.runner.ansible_runner_instance.rc
+            # Sleep briefly to prevent 100% CPU utilization
+            # in mode stdout, the delay introduced by the curses key read is not present
+            time.sleep(0.01)
+        return_code = self.runner.ansible_runner_instance.rc
+        if return_code != 0:
+            return RunStdoutReturn(
+                message="Please review the log for errors.",
+                return_code=return_code,
+            )
+        return RunStdoutReturn(message="", return_code=return_code)
 
-    def run(self, interaction: Interaction, app: AppPublic) -> Union[Interaction, None]:
-        # pylint: disable=too-many-branches
-        """run :run or :replay
+    def run(self, interaction: Interaction, app: AppPublic) -> Optional[Interaction]:
+        """Run :run or :replay.
 
         :param interaction: The interaction from the user
-        :type interaction: Interaction
         :param app: The app instance
-        :type app: App
+        :returns: The pending interaction or none
         """
-
         self._prepare_to_run(app, interaction)
 
         if interaction.action.match.groupdict().get("run"):
@@ -286,6 +306,14 @@ class Action(App):
 
         self.steps.append(self._plays)
 
+        # Show a notification until the first the first message from the queue is processed
+        if self._subaction_type == "run":
+            messages = ["Preparing for automation, please wait..."]
+            notification = nonblocking_notification(messages=messages)
+            interaction.ui.show_form(notification)
+            while not self._first_message_received:
+                self.update()
+
         while True:
             self.update()
 
@@ -297,7 +325,8 @@ class Action(App):
                     self.steps.append(self._plays)
                 else:
                     self._logger.debug(
-                        "No steps remaining for '%s' returning to calling app", self._name
+                        "No steps remaining for '%s' returning to calling app",
+                        self._name,
                     )
                     break
 
@@ -316,26 +345,24 @@ class Action(App):
 
     # pylint: disable=too-many-branches
     def _init_run(self) -> bool:
-        """in the case of :run, check the user input"""
+        """In the case of :run, check the user input.
 
+        :returns: True
+        """
         # Ensure the playbook and inventory are valid
 
-        self._update_args(
-            ["run"] + shlex.split(self._interaction.action.match.groupdict()["params_run"] or "")
+        args_updated = self._update_args(
+            ["run"] + shlex.split(self._interaction.action.match.groupdict()["params_run"] or ""),
         )
+        if not args_updated:
+            return False
 
         if isinstance(self._args.playbook, str):
             playbook_valid = os.path.exists(self._args.playbook)
         else:
             playbook_valid = False
 
-        if isinstance(self._args.inventory, list):
-            inventory_valid = all((os.path.exists(inv) for inv in self._args.inventory))
-        else:
-            # Permit running without an inventory
-            inventory_valid = True
-
-        if not all((playbook_valid, inventory_valid)):
+        if not playbook_valid:
 
             populated_form = self._prompt_for_playbook()
             if populated_form["cancelled"]:
@@ -343,32 +370,35 @@ class Action(App):
 
             new_cmd = ["run"]
             new_cmd.append(populated_form["fields"]["playbook"]["value"])
-            for field in populated_form["fields"].values():
-                if field["name"].startswith("inv_") and field["value"] != "":
-                    new_cmd.extend(["-i", field["value"]])
+
             if populated_form["fields"]["cmdline"]["value"]:
                 new_cmd.extend(shlex.split(populated_form["fields"]["cmdline"]["value"]))
 
             # Parse as if provided from the cmdline
-            self._update_args(new_cmd)
+            args_updated = self._update_args(new_cmd)
+            if not args_updated:
+                return False
 
         self._run_runner()
         self._logger.info("Run initialized and playbook started.")
         return True
 
     def _init_replay(self) -> bool:
-        """in the case of :replay, replay the artifact
-        check for a version, to be safe
-        copy the calling app args as our our so the can be updated safely
-        with a uuid attached to the name
+        """In the case of :replay, replay the artifact.
+
+        Check for a version and process artifact file.
+
+        :returns: True if replay completes, False if there is an error
         """
         self._logger.debug("Starting replay artifact request with mode %s", self.mode)
 
         if self.mode == "interactive":
-            self._update_args(
+            args_updated = self._update_args(
                 ["replay"]
-                + shlex.split(self._interaction.action.match.groupdict()["params_replay"] or "")
+                + shlex.split(self._interaction.action.match.groupdict()["params_replay"] or ""),
             )
+            if not args_updated:
+                return False
 
         artifact_file = self._args.playbook_artifact_replay
 
@@ -384,15 +414,15 @@ class Action(App):
             artifact_file = populated_form["fields"]["artifact_file"]["value"]
 
         try:
-            with open(artifact_file) as json_file:
-                data = json.load(json_file)
+            with open(artifact_file, encoding="utf-8") as fh:
+                data = json.load(fh)
         except json.JSONDecodeError as exc:
             self._logger.debug("json decode error: %s", str(exc))
             self._logger.error("Unable to parse artifact file")
             return False
 
         version = data.get("version", "")
-        if version.startswith("1."):
+        if version.startswith("1.") or version.startswith("2."):
             try:
                 stdout = data["stdout"]
                 if self.mode == "interactive":
@@ -411,7 +441,8 @@ class Action(App):
                 return False
         else:
             self._logger.error(
-                "Incompatible artifact version, got '%s', compatible = '1.y.z'", version
+                "Incompatible artifact version, got '%s', compatible = '1.y.z'",
+                version,
             )
             return False
 
@@ -420,8 +451,11 @@ class Action(App):
         return True
 
     def _prompt_for_artifact(self, artifact_file: str) -> Dict[Any, Any]:
-        """prompt for a valid artifact file"""
+        """Prompt for a valid artifact file.
 
+        :param artifact_file: Artifact file
+        :returns: Dict with artifact detail entries
+        """
         if not isinstance(artifact_file, str):
             artifact_file = ""
 
@@ -439,13 +473,15 @@ class Action(App):
         }
         form_dict["fields"].append(form_field)
         form = dict_to_form(form_dict)
-        self._interaction.ui.show(form)
+        self._interaction.ui.show_form(form)
         populated_form = form_to_dict(form, key_on_name=True)
         return populated_form
 
     def _prompt_for_playbook(self) -> Dict[Any, Any]:
-        """prepopulate a form to confirm the playbook details"""
+        """Prepopulate a form to confirm the playbook details.
 
+        :returns: Dict with playbook and inventory detail entries
+        """
         self._logger.debug("Inventory/Playbook not set, provided, or valid, prompting")
 
         if isinstance(self._args.playbook, str):
@@ -453,11 +489,6 @@ class Action(App):
         else:
             playbook = ""
 
-        if isinstance(self._args.inventory, list):
-            inventory = self._args.inventory
-        else:
-            inventory = ["", "", ""]
-
         if isinstance(self._args.cmdline, list):
             cmdline = " ".join(self._args.cmdline)
         else:
@@ -465,7 +496,7 @@ class Action(App):
 
         FType = Dict[str, Any]
         form_dict: FType = {
-            "title": "Inventory and/or playbook not found, please confirm the following",
+            "title": "Playbook not found, please confirm the following",
             "fields": [],
         }
         form_field = {
@@ -477,16 +508,6 @@ class Action(App):
         }
         form_dict["fields"].append(form_field)
 
-        for idx, inv in enumerate(inventory):
-            form_field = {
-                "name": f"inv_{idx}",
-                "pre_populate": inv,
-                "prompt": "Inventory source",
-                "type": "text_input",
-                "validator": {"name": "none"},
-            }
-            form_dict["fields"].append(form_field)
-
         form_field = {
             "name": "cmdline",
             "pre_populate": cmdline,
@@ -496,13 +517,12 @@ class Action(App):
         }
         form_dict["fields"].append(form_field)
         form = dict_to_form(form_dict)
-        self._interaction.ui.show(form)
+        self._interaction.ui.show_form(form)
         populated_form = form_to_dict(form, key_on_name=True)
         return populated_form
 
     def _take_step(self) -> None:
-        """run the current step on the stack"""
-
+        """Run the current step on the stack."""
         result = None
         if isinstance(self.steps.current, Interaction):
             result = run_action(self.steps.current.name, self.app, self.steps.current)
@@ -542,7 +562,10 @@ class Action(App):
             self.steps.append(result)
 
     def _run_runner(self) -> None:
-        """spin up runner"""
+        """Spin up runner.
+
+        :raises RuntimeError: If no ansible-playbook executable
+        """
         executable_cmd: Optional[str]
 
         if self.mode == "stdout_w_artifact":
@@ -551,12 +574,12 @@ class Action(App):
             mode = self.mode
 
         if isinstance(self._args.set_environment_variable, dict):
-            set_envvars = {**self._args.set_environment_variable}
+            set_env_vars = {**self._args.set_environment_variable}
         else:
-            set_envvars = {}
+            set_env_vars = {}
 
         if self._args.display_color is False:
-            set_envvars["ANSIBLE_NOCOLOR"] = "1"
+            set_env_vars["ANSIBLE_NOCOLOR"] = "1"
 
         kwargs = {
             "container_engine": self._args.container_engine,
@@ -566,7 +589,7 @@ class Action(App):
             "inventory": self._args.inventory,
             "navigator_mode": mode,
             "pass_environment_variable": self._args.pass_environment_variable,
-            "set_environment_variable": set_envvars,
+            "set_environment_variable": set_env_vars,
             "private_data_dir": self._args.ansible_runner_artifact_dir,
             "rotate_artifacts": self._args.ansible_runner_rotate_artifacts_count,
             "timeout": self._args.ansible_runner_timeout,
@@ -577,7 +600,7 @@ class Action(App):
 
         if isinstance(self._args.execution_environment_volume_mounts, list):
             kwargs.update(
-                {"container_volume_mounts": self._args.execution_environment_volume_mounts}
+                {"container_volume_mounts": self._args.execution_environment_volume_mounts},
             )
 
         if isinstance(self._args.container_options, list):
@@ -599,15 +622,17 @@ class Action(App):
             pass_through_arg.extend(self._args.cmdline)
         kwargs.update({"cmdline": pass_through_arg})
 
-        self.runner = CommandRunnerAsync(executable_cmd=executable_cmd, queue=self._queue, **kwargs)
+        self.runner = CommandAsync(executable_cmd=executable_cmd, queue=self._queue, **kwargs)
         self.runner.run()
         self._runner_finished = False
         self._logger.debug("runner requested to start")
 
     def _dequeue(self) -> None:
-        """Drain the runner queue"""
+        """Drain the runner queue."""
         drain_count = 0
         while not self._queue.empty():
+            if not self._first_message_received:
+                self._first_message_received = True
             message = self._queue.get()
             self._handle_message(message)
             drain_count += 1
@@ -616,104 +641,156 @@ class Action(App):
 
     def _handle_message(self, message: dict) -> None:
         # pylint: disable=too-many-branches
-        # pylint: disable=too-many-nested-blocks
         # pylint: disable=too-many-statements
-        """Handle a runner message
+        # pylint: disable=too-many-locals
+        # pylint: disable=too-many-return-statements
+        """Handle a runner message.
 
         :param message: The message from runner
         :type message: dict
         """
+        # Collect any stdout
+        if "stdout" in message and message["stdout"]:
+            self.stdout.extend(message["stdout"].splitlines())
+            if self.mode == "stdout_w_artifact":
+                print(message["stdout"])
+
+        # Get the event data
         try:
             event = message["event"]
+            event_data = message["event_data"]
         except KeyError:
             error = f"Unhandled message from runner queue, discarded: {message}"
             self._logger.critical(error)
+            return
+
+        # Handle verbose and error messages
+        if event in ["verbose", "error"]:
+            if "ERROR!" in message["stdout"]:
+                self._msg_from_plays = ("ERROR", 9)
+                if self.mode == "interactive":
+                    self._notify_error(message["stdout"])
+            elif "WARNING" in message["stdout"]:
+                self._msg_from_plays = ("WARNINGS", 13)
+            return
+
+        if event == "playbook_on_play_start":
+            event_data["__play_name"] = event_data["name"]
+            event_data["tasks"] = []
+            self._plays.value.append(event_data)
+            return
+
+        if event == "playbook_on_task_start":
+            self._task_cache[event_data["task_uuid"]] = event_data["task"]
+            return
+
+        # Only runner on_* events are relevant now
+        try:
+            prefix, runner_event = event.rsplit("_", 1)
+            assert prefix == "runner_on"
+            assert runner_event in ("ok", "skipped", "start", "unreachable", "failed")
+        except (AssertionError, ValueError):
+            return
+
+        # Find the parent play of the task
+        try:
+            play = next(p for p in self._plays.value if p["uuid"] == event_data["play_uuid"])
+        except StopIteration:
+            self._logger.warning("Playbook event without parent play")
+            return
+
+        # New task encountered
+        if runner_event == "start":
+            try:
+                previous_name = self._task_cache[event_data["task_uuid"]]
+                use_previous = not is_jinja(previous_name)
+            except KeyError:
+                use_previous = False
+
+            event_data.update(
+                {
+                    "__changed": "unknown",
+                    "__duration": "Pending",
+                    "__host": event_data["host"],
+                    "__number": len(play["tasks"]),
+                    "__result": "In progress",
+                    "__task_action": event_data["task_action"],
+                    "__task": previous_name if use_previous else event_data["task"],
+                },
+            )
+            play["tasks"].append(event_data)
+            return
+
+        # The runner event indicates a task has finished, find the task in the play
+        match = ("task_uuid", "host")
+        try:
+            task = next(
+                t for t in play["tasks"] if itemgetter(*match)(t) == itemgetter(*match)(event_data)
+            )
+        except StopIteration:
+            self._logger.warning("Task event without parent task")
+            return
+
+        # Get the duration
+        if isinstance(event_data["duration"], (int, float)):
+            duration = human_time(seconds=round_half_up(event_data["duration"]))
         else:
-            if "stdout" in message and message["stdout"]:
-                self.stdout.extend(message["stdout"].splitlines())
-                if self.mode == "stdout_w_artifact":
-                    print(message["stdout"])
+            self._logger.debug(
+                "Task duration for '%s' was type '%s', set to 0",
+                event_data["task"],
+                type(event_data["duration"]),
+            )
+            duration = human_time(seconds=0)
 
-            if event in ["verbose", "error"]:
-                if "ERROR!" in message["stdout"]:
-                    self._msg_from_plays = ("ERROR", 9)
-                    if self.mode == "interactive":
-                        self._notify_error(message["stdout"])
-                elif "WARNING" in message["stdout"]:
-                    self._msg_from_plays = ("WARNINGS", 13)
+        # Replace failed with ignored
+        modify_result = runner_event == "failed" and event_data["ignore_errors"]
 
-            if event == "playbook_on_play_start":
-                play = message["event_data"]
-                play["__play_name"] = play["name"]
-                play["tasks"] = []
-                self._plays.value.append(play)
+        event_data.update(
+            {
+                "__changed": event_data.get("res", {}).get("changed", False),
+                "__duration": duration,
+                "__result": ("ignored" if modify_result else runner_event).capitalize(),
+            },
+        )
 
-            if event.startswith("runner_on_"):
-                runner_event = event.split("_")[2]
-                task = message["event_data"]
-                play_id = next(
-                    idx for idx, p in enumerate(self._plays.value) if p["uuid"] == task["play_uuid"]
-                )
-                if runner_event in ["ok", "skipped", "unreachable", "failed"]:
-                    if runner_event == "failed" and task["ignore_errors"]:
-                        result = "ignored"
-                    else:
-                        result = runner_event
-                    task["__result"] = result.upper()
-                    task["__changed"] = task.get("res", {}).get("changed", False)
-                    task["__duration"] = human_time(seconds=round(task["duration"], 2))
-                    task_id = None
-                    for idx, play_task in enumerate(self._plays.value[play_id]["tasks"]):
-                        if task["task_uuid"] == play_task["task_uuid"]:
-                            if task["host"] == play_task["host"]:
-                                task_id = idx
-                                break
-                    if task_id is not None:
-                        self._plays.value[play_id]["tasks"][task_id].update(task)
+        # Find the best name for the task
+        name_changed = task["__task"] != event_data["task"]
+        no_longer_templated = is_jinja(task["__task"]) and not is_jinja(event_data["task"])
+        changed_and_not_templated = name_changed and not is_jinja(event_data["task"])
+        if no_longer_templated or changed_and_not_templated:
+            event_data["__task"] = event_data["task"]
 
-                elif runner_event == "start":
-                    task["__host"] = task["host"]
-                    task["__result"] = "IN_PROGRESS"
-                    task["__changed"] = "unknown"
-                    task["__duration"] = None
-                    task["__number"] = len(self._plays.value[play_id]["tasks"])
-                    task["__task"] = task["task"]
-                    task["__task_action"] = task["task_action"]
-                    self._plays.value[play_id]["tasks"].append(task)
+        task.update(event_data)
 
     def _play_stats(self) -> None:
-        """Calculate the play's stats based
-        on it's tasks
-        """
+        """Calculate the play's stats based on it's tasks."""
         for idx, play in enumerate(self._plays.value):
-            total = ["__ok", "__skipped", "__failed", "__unreachable", "__ignored", "__in_progress"]
+            total = ["__ok", "__skipped", "__failed", "__unreachable", "__ignored", "__in progress"]
             self._plays.value[idx].update(
                 {
                     tot: len([t for t in play["tasks"] if t["__result"].lower() == tot[2:]])
                     for tot in total
-                }
+                },
             )
             self._plays.value[idx]["__changed"] = len(
-                [t for t in play["tasks"] if t["__changed"] is True]
+                [t for t in play["tasks"] if t["__changed"] is True],
             )
             task_count = len(play["tasks"])
             self._plays.value[idx]["__task_count"] = task_count
-            completed = task_count - self._plays.value[idx]["__in_progress"]
+            completed = task_count - self._plays.value[idx]["__in progress"]
             if completed:
                 new = floor((completed / task_count * 100))
-                current = self._plays.value[idx].get("__pcomplete", 0)
-                self._plays.value[idx]["__pcomplete"] = max(new, current)
+                current = self._plays.value[idx].get("__percent_complete", 0)
+                self._plays.value[idx]["__percent_complete"] = max(new, current)
                 self._plays.value[idx]["__progress"] = str(max(new, current)) + "%"
             else:
                 self._plays.value[idx]["__progress"] = "0%"
 
     def _prepare_to_quit(self, interaction: Interaction) -> bool:
-        """Looks like we're headed out of here
+        """Pre-quit tasks.
 
-        :param interaction: the quit interaction
-        :type interaction: Interaction
-        :return: a bool indicating whether of not it's safe to exit
-        :rtype: bool
+        :param interaction: The quit interaction
+        :returns: A bool indicating whether or not it's safe to exit
         """
         self.update()
         if self.runner is not None and not self.runner.finished:
@@ -730,15 +807,14 @@ class Action(App):
         return True
 
     def _task_list_for_play(self) -> Step:
-        """generate a menu of task for the currently selected play
+        """Generate a menu of tasks for the currently selected play.
 
-        :return: The menu step
-        :rtype: Step
+        :returns: The menu step
         """
         value = self.steps.current.selected["tasks"]
         step = Step(
             name="task_list",
-            tipe="menu",
+            step_type="menu",
             columns=self._task_list_columns,
             select_func=self._task_from_task_list,
             value=value,
@@ -746,19 +822,17 @@ class Action(App):
         return step
 
     def _task_from_task_list(self) -> Step:
-        """generate task content for the selected task
+        """Generate task content for the selected task.
 
-        :return: content whic show a task
-        :rtype: Step
+        :returns: Content which shows a task
         """
         value = self.steps.current.value
         index = self.steps.current.index
-        step = Step(name="task", tipe="content", index=index, value=value)
+        step = Step(name="task", step_type="content", index=index, value=value)
         return step
 
     def update(self) -> None:
-        """Drain the queue, set the status and write the artifact if needed"""
-
+        """Drain the queue, set the status and write the artifact if needed."""
         # let the calling app update as well
         self._calling_app.update()
 
@@ -767,16 +841,16 @@ class Action(App):
             self._set_status()
 
             if self.runner.finished and not self._runner_finished:
-                # self._interaction.ui.disable_refresh()
                 self._logger.debug("runner finished")
                 self._logger.info("Playbook complete")
                 self.write_artifact()
                 self._runner_finished = True
+                self._notify_no_tasks_redirect()
 
     def _get_status(self) -> Tuple[str, int]:
-        """Get the status and color
+        """Get the runner status and color for status message.
 
-        :return: status string, status color
+        :returns: status string, status color
         """
         status = ""
         status_color = 0
@@ -797,12 +871,12 @@ class Action(App):
         return status, status_color
 
     def _set_status(self) -> None:
-        """Set the ui status"""
+        """Set the UI status."""
         status, status_color = self._get_status()
         self._interaction.ui.update_status(status, status_color)
 
     def write_artifact(self, filename: Optional[str] = None) -> None:
-        """Write the artifact
+        """Write the artifact.
 
         :param filename: The file to write to
         :type filename: str
@@ -816,7 +890,7 @@ class Action(App):
             filename = filename.format(
                 playbook_dir=os.path.dirname(self._args.playbook),
                 playbook_name=os.path.splitext(os.path.basename(self._args.playbook))[0],
-                ts_utc=datetime.datetime.now(tz=datetime.timezone.utc).isoformat(),
+                time_stamp=now_iso(self._args.time_zone),
             )
             self._logger.debug("Formatted artifact file name set to %s", filename)
             filename = abs_user_path(filename)
@@ -826,16 +900,23 @@ class Action(App):
 
             try:
                 os.makedirs(os.path.dirname(filename), exist_ok=True)
-                with open(filename, "w") as outfile:
-                    artifact = {
-                        "version": "1.0.0",
-                        "plays": self._plays.value,
-                        "stdout": self.stdout,
-                        "status": status,
-                        "status_color": status_color,
-                    }
-                    json.dump(artifact, outfile, indent=4)
-                    self._logger.info("Saved artifact as %s", filename)
+                artifact = {
+                    "version": "2.0.0",
+                    "plays": self._plays.value,
+                    "stdout": self.stdout,
+                    "status": status,
+                    "status_color": status_color,
+                    "settings_entries": to_effective(self._args),
+                    "settings_sources": to_sources(self._args),
+                }
+                serialize_write_file(
+                    content=artifact,
+                    content_view=ContentView.NORMAL,
+                    file_mode="w",
+                    file=Path(filename),
+                    serialization_format=SerializationFormat.JSON,
+                )
+                self._logger.info("Saved artifact as %s", filename)
 
             except (IOError, OSError) as exc:
                 error = (
@@ -844,9 +925,10 @@ class Action(App):
                 self._logger.error(error)
 
     def rerun(self) -> None:
-        """rerun the current playbook
-        since we're not reinstantiating run,
-        drain the queue, clear the steps, reset the index, etc
+        """Rerun the current playbook.
+
+        Since we're not reinstating run, drain the queue,
+        clear the steps, reset the index, etc.
         """
         if self._subaction_type == "run":
             if self.runner.finished:
@@ -867,7 +949,10 @@ class Action(App):
             self._logger.error("sub-action type '%s' is invalid", self._subaction_type)
 
     def _notify_error(self, message: str):
-        """show a blocking warning"""
+        """Show a blocking warning.
+
+        :param message: Message for warning
+        """
         warn_msg = ["Errors were encountered while running the playbook:"]
         messages = remove_ansi(message).splitlines()
         messages[-1] += "..."
@@ -875,4 +960,17 @@ class Action(App):
         warn_msg += ["[HINT] To see the full error message try ':stdout'"]
         warn_msg += ["[HINT] After it's fixed, try to ':rerun' the playbook"]
         warning = warning_notification(warn_msg)
-        self._interaction.ui.show(warning)
+        self._interaction.ui.show_form(warning)
+
+    def _notify_no_tasks_redirect(self):
+        """In the case the playbook finished but without tasks, show a warning, send to stdout."""
+        total_tasks = sum(len(play["tasks"]) for play in self._plays.value)
+        # At least one task, no need to redirect
+        if total_tasks:
+            return
+
+        message = ["The playbook completed without tasks. Redirecting to ':stdout' for review."]
+        warning = warning_notification(message)
+
+        self._interaction.ui.show_form(warning)
+        self.steps.append(Interaction(name="stdout", action=stdout_action, ui=self._interaction.ui))
