@@ -1,17 +1,21 @@
+from typing import TYPE_CHECKING
 from typing import List
 from typing import Tuple
-from typing import TYPE_CHECKING
 
 from .region import Region
 from .region import Regions
 from .state import State
 
+
 if TYPE_CHECKING:
     from .compiler import Compiler
 
 
 def tokenize(
-    compiler: "Compiler", state: State, line: str, first_line: bool
+    compiler: "Compiler",
+    state: State,
+    line: str,
+    first_line: bool,
 ) -> Tuple[State, Regions]:
 
     """tokenize a string into it's parts"""
